package dk.nsp.idsas.security.impl;

import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.IdsasActorValidator;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;

public class IdsasActorValidatorImpl implements IdsasActorValidator {

    private final WhitelistDao whitelistDao;

    public IdsasActorValidatorImpl(WhitelistDao whitelistDao) {
        this.whitelistDao = whitelistDao;
    }

    @Override
    public IdsasActor validateIdsasActor(IdsasActor actor, WhitelistType whitelistType) throws IdsasSecurityException {
        validateCvrWhitelistedForType(actor, whitelistType);
        return actor;
    }

    private void validateCvrWhitelistedForType(IdsasActor actor, WhitelistType whitelistType) throws IdsasSecurityException {
        if (!whitelistDao.isWhitelisted(actor.getCvr(), whitelistType)) {
            throw new IdsasSecurityException(ErrorMessages.whitelistError(actor.getCvr(), whitelistType));
        }
    }
}
