package dk.nsp.idsas.ws;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.sql.DataSource;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;

import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.util.WhitelistType;

public abstract class IdsasAbstractWS {
	public static final String ANNOTATION_CONFIG_CONTEXT = "ANNOTATION_CONFIG_APPLICATION_CONTEXT";

	@Resource
	protected WebServiceContext context;

	@Autowired
	protected IdsasActorBuilder idsasActorBean;

	@Autowired
	protected DataSource dataSource;

	@PostConstruct
	public void init() {
		ServletContext servletContext = (ServletContext) context.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
		Object o = servletContext.getAttribute(ANNOTATION_CONFIG_CONTEXT);
		((AnnotationConfigApplicationContext)o).getAutowireCapableBeanFactory().autowireBean(this);
	}

	protected SLALogItem getSlaLogItemFromContext() {
		ServletRequest servletRequest = (ServletRequest) context.getMessageContext().get(MessageContext.SERVLET_REQUEST);
		return (SLALogItem) servletRequest.getAttribute(SLALoggingServletFilter.CURRENT_LOGITEM_REQUEST_KEY);
	}

	protected IdsasActor getValidatedIdsasActor(WhitelistType whitelistType) {
		try {
			return idsasActorBean.getValidatedActor(whitelistType);
		} catch (IdsasSecurityException e) {
			throw SoapFaultGenerator.generateSOAPFault(true, e.getMessage());
		}
	}

	protected IdsasActor getValidatedIdsasActor() {
		try {
			return idsasActorBean.getValidatedActor();
		} catch (IdsasSecurityException e) {
			throw SoapFaultGenerator.generateSOAPFault(true, e.getMessage());
		}
	}
}
