package dk.nsp.idsas.security;

import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.impl.IdsasActorValidatorImpl;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class IdsasActorValidatorTest {

    private IdsasActorValidatorImpl idsasActorValidator;
    private WhitelistDao whitelistDao;

    @Before
    public void setUp() throws Exception {
        whitelistDao = Mockito.mock(WhitelistDao.class);
        idsasActorValidator = new IdsasActorValidatorImpl(whitelistDao);
    }

    @Test
    public void validateIdsasActorAndGetContext_ForWhitelistTypeSaltWhenCvrIsWhitelisted_ReturnsContextWithActor() throws IdsasSecurityException {
        String cvr = "12345678";
        IdsasActor actorInput = IdsasActor.createSystem(cvr);
        Mockito.when(whitelistDao.isWhitelisted(cvr, WhitelistType.SALT)).thenReturn(true);

        IdsasActor actor = idsasActorValidator.validateIdsasActor(actorInput, WhitelistType.SALT);

        assertEquals(actorInput, actor);
    }

    @Test
    public void validateIdsasActorAndGetContext_ForWhitelistTypeSaltWhenCvrIsNotWhitelisted_ThrowsIdsasSecurityException() {
        String cvr = "12345678";
        IdsasActor actor = IdsasActor.createSystem(cvr);

        assertThrows(IdsasSecurityException.class, () -> idsasActorValidator.validateIdsasActor(actor, WhitelistType.SALT));
    }

    @Test
    public void validateIdsasActorAndGetContext_ForWhitelistTypeBlurringWhenCvrIsWhitelisted_ReturnsContextWithActor() throws IdsasSecurityException {
        String cvr = "12345678";
        IdsasActor actorInput = IdsasActor.createSystem(cvr);
        Mockito.when(whitelistDao.isWhitelisted(cvr, WhitelistType.BLURRING)).thenReturn(true);

        IdsasActor actor = idsasActorValidator.validateIdsasActor(actorInput, WhitelistType.BLURRING);

        assertEquals(actorInput, actor);
    }

    @Test
    public void validateIdsasActorAndGetContext_ForWhitelistTypeBlurringWhenCvrIsNotWhitelisted_ThrowsIdsasSecurityException() {
        String cvr = "12345678";
        IdsasActor actor = IdsasActor.createSystem(cvr);

        assertThrows(IdsasSecurityException.class, () -> idsasActorValidator.validateIdsasActor(actor, WhitelistType.BLURRING));
    }
}
