package dk.nsp.idsas.test.stepdefinitions;

import javax.xml.datatype.DatatypeConfigurationException;
import java.net.URI;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Value;

import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.test.idp.CitizenIdentities;
import dk.nsp.test.idp.EmployeeIdentities;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;

import static dk.nsp.idsas.test.util.Util.toXmlGregorian;
import static org.junit.Assert.*;

public class BlurringStepDefinitions {

    @Value("${idsas.service.endpoint}" + "/20230601/service")
    private String idsasServiceUrl;

    @Value("${idsas.service-lookup.endpoint}" + "/20230601/lookup")
    private String idsasServiceLookupUrl;

    private final ServiceCaller serviceCaller = new ServiceCaller();

    private String response = "";

    private Identity lookupTestIdentity = OrganizationIdentities.testOrganisation98021838();
    private String lookupTestIdentityCVR = "98021838";

    @When("en {string} opretter en slørring for en borger med CPR {string}")
    public void enOpretterEnSlørringForEnBorgerMedCPR(String brugertype, String cpr) throws DatatypeConfigurationException {
        opretSlørring(brugertype, cpr, LocalDateTime.now().plusDays(1));
    }

    @Then("så bliver slørringen oprettet")
    public void såBliverSlørringenOprettet() {
        assertEquals("OK", response);
    }

    @Then("så fejler oprettelsen af slørringen med forkert brugertype")
    public void såFejlerOprettelsenAfSlørringen() {
        assertTrue(response.contains(ErrorMessages.invalidUserType()));
    }

    @When("en {string} opretter en slørring for en borger med CPR {string}, som udløb i går")
    public void enOpretterEnSlørringForEnBorgerMedCPRSomUdløberIGår(String brugertype, String cpr) throws DatatypeConfigurationException {
        // Slørringen udløber i går
        opretSlørring(brugertype, cpr, LocalDateTime.now().minusDays(1));
    }

    @Then("så har {string} ikke længere aktuelle slørring over for det cvr-nummer")
    public void såHarIkkeLængereAktuelleSlørringOverForDetCvrNummer(String cpr) {
        List<String> result = lookupSlørring(cpr);
        assertFalse(result.contains(lookupTestIdentityCVR));
    }

    private List<String> lookupSlørring(String cpr) {
        GetBlurredOrganisationsRequestType request = new GetBlurredOrganisationsRequestType();
        PatientIdType patientId = new PatientIdType();
        BlurredOrganisationsResponseType response;

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);

        request.setPatientId(patientId);
        try {
            response = serviceCaller.callRoot(lookupTestIdentity,
                    request, SOAPOperation.GetBlurredOrganisations, new URI(idsasServiceLookupUrl),
                    GetBlurredOrganisationsRequestType.class,
                    BlurredOrganisationsResponseType.class);
            return response.getOrganisationIds().stream().map(OrganisationIdType::getId).collect(Collectors.toList());
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }

        return null;
    }

    private void opretSlørring(String brugertype, String cpr, LocalDateTime dateTime) throws DatatypeConfigurationException {
        CreateBlurringRequestType request = new CreateBlurringRequestType();
        PatientIdType patientId = new PatientIdType();

        // Default brugertype: "system"
        Identity identity = OrganizationIdentities.testOrganisation98021838();
        if (brugertype.equals("sundhedsfaglig")) {
            identity = EmployeeIdentities.lægeCharlesBabbage();
        } else if (brugertype.equals("borger")) {
            identity = CitizenIdentities.adaLovelace("https://fmk");
        }

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);
        request.setPatientId(patientId);
        request.setEndDateTime(toXmlGregorian(dateTime));
        try {
            response = serviceCaller.callRoot(identity,
                    request, SOAPOperation.CreateBlurring, new URI(idsasServiceUrl),
                    CreateBlurringRequestType.class,
                    String.class);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

}