package dk.nsp.idsas.setup;

import javax.sql.DataSource;

import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.*;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.security.impl.IdsasActorValidatorImpl;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;

import dk.nsp.idsas.dao.AlivenessDao;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.AuditLogger;

@Configuration
@PropertySources({
        @PropertySource(value = "idsas-lookup.properties", encoding = "ISO-8859-1")
})
public class ServiceSetup {
    public static final String ANNOTATION_CONFIG_CONTEXT = "ANNOTATION_CONFIG_APPLICATION_CONTEXT";
    private static final Logger logger = Logger.getLogger(ServiceSetup.class);

    @Value("${idsas.datasource.jndi}")
    private String dataSourceName;

    @Value("${idsas.client.system:http://system.nspop.dk/sts}")
    private String clientSystemName;

    @Bean
    public DataSource dataSource() {
        return new JndiDataSourceLookup().getDataSource(dataSourceName);
    }

    @Bean
    public AlivenessDao alivenessDao(DataSource dataSource) {
        return new AlivenessDao(dataSource);
    }

    @Bean WhitelistDao whitelistDao(DataSource dataSource) {
        return new WhitelistDao(dataSource);
    }

    @Bean
    public IdsasActorBuilder idsasActorBuilder(WhitelistDao whitelistDao) {
        IdsasActorProvider provider = new IdsasActorProviderImpl();
        IdsasActorTransformer transformer = actor -> actor; // No transforming
        IdsasActorEnricher enricher = actor -> actor; // No enriching
        IdsasActorValidator validator = new IdsasActorValidatorImpl(whitelistDao);
        return new IdsasActorBuilder(provider, transformer, enricher, validator);
    }

    @Bean
    public LookupService lookupService() {
        return new LookupService();
    }

    @Bean
    public AuditLogger auditLogger() {
        return new AuditLogger();
    }

    public String getClientSystemName() {
        return clientSystemName;
    }
}
