package dk.nsp.idsas.ws._20230601;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;


import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import dk.sds.nsp.audit.Audit;

import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;

import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.setup.ServiceSetup;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasLookupService", urlPatterns={"/20230601/lookup"})
@WebService(serviceName = "IdsasLookupService", wsdlLocation = "wsdl/20230601/idsas-lookup.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasLookupServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
public class IdsasLookupServiceWS extends IdsasAbstractWS implements IdsasLookupServicePortType {
    private static final Logger logger = Logger.getLogger(IdsasLookupServiceWS.class);

    @Autowired
    private LookupService lookupService;

    @Autowired
    private AuditLogger auditLogger;

    @Autowired
    private ServiceSetup setup;

    @Override
    public BlurredOrganisationsResponseType getBlurredOrganisations(GetBlurredOrganisationsRequestType getBlurredOrganisationsRequest) {

        IdsasActor actor = getValidatedIdsasActor();
        String patientCpr = getBlurredOrganisationsRequest.getPatientId().getId();
        String patientIdType = getBlurredOrganisationsRequest.getPatientId().getClassification().name();
        logger.info(String.format("Enter get blurred organisations for patient %s", patientCpr));

        // Validering af, at actor er en systembruger
        if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw SoapFaultGenerator.generateSOAPFault(true, ErrorMessages.userTypNotSupported(actor.getActorType().name()));
        }

        // Validering af, at systembrugeren har et clientName med den korrekte værdi
        if (!actor.getClientSystemName().equals(setup.getClientSystemName())) {
            throw SoapFaultGenerator.generateSOAPFault(true,
                    ErrorMessages.clientSystemNameError(actor.getClientSystemName(), setup.getClientSystemName())
            );
        }

        // Tjek at patientId har typen CPR; vi håndterer ikke andre typer lige nu
        String idClassification = getBlurredOrganisationsRequest.getPatientId().getClassification() == null ?
                "null" : getBlurredOrganisationsRequest.getPatientId().getClassification().value();
        if (!idClassification.equals(PatientIdClassificationType.CPR.value())) {
            throw SoapFaultGenerator.generateSOAPFault(true,
                    ErrorMessages.idTypeNotSupportedError(idClassification));
        }

        // Liste af blurrings hentes
        BlurredOrganisationsResponseType entityList = lookupService.getActiveBlurrings(patientCpr, patientIdType);

        // Audit logning
        auditLogger.logGetBlurredOrganisations(Audit.createAuditBuilder(), patientCpr);

        return entityList;
    }
}
